/********************************************************************************
** Form generated from reading UI file 'svnprogressdialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SVNPROGRESSDIALOG_H
#define UI_SVNPROGRESSDIALOG_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QPlainTextEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_SvnProgressDialog
{
public:
    QGridLayout *gridLayout;
    QPlainTextEdit *texteditMessage;
    QPushButton *buttonCancel;
    QPushButton *buttonOk;
    QSpacerItem *horizontalSpacer;

    void setupUi(QWidget *SvnProgressDialog)
    {
        if (SvnProgressDialog->objectName().isEmpty())
            SvnProgressDialog->setObjectName("SvnProgressDialog");
        SvnProgressDialog->resize(521, 409);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Minimum);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(SvnProgressDialog->sizePolicy().hasHeightForWidth());
        SvnProgressDialog->setSizePolicy(sizePolicy);
        gridLayout = new QGridLayout(SvnProgressDialog);
        gridLayout->setObjectName("gridLayout");
        texteditMessage = new QPlainTextEdit(SvnProgressDialog);
        texteditMessage->setObjectName("texteditMessage");
        texteditMessage->setReadOnly(true);

        gridLayout->addWidget(texteditMessage, 0, 0, 1, 3);

        buttonCancel = new QPushButton(SvnProgressDialog);
        buttonCancel->setObjectName("buttonCancel");
        QIcon icon(QIcon::fromTheme(QString::fromUtf8("dialog-cancel")));
        buttonCancel->setIcon(icon);

        gridLayout->addWidget(buttonCancel, 1, 2, 1, 1);

        buttonOk = new QPushButton(SvnProgressDialog);
        buttonOk->setObjectName("buttonOk");
        buttonOk->setEnabled(false);
        QIcon icon1(QIcon::fromTheme(QString::fromUtf8("dialog-ok")));
        buttonOk->setIcon(icon1);
        buttonOk->setAutoDefault(true);

        gridLayout->addWidget(buttonOk, 1, 1, 1, 1);

        horizontalSpacer = new QSpacerItem(328, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(horizontalSpacer, 1, 0, 1, 1);


        retranslateUi(SvnProgressDialog);

        buttonOk->setDefault(true);


        QMetaObject::connectSlotsByName(SvnProgressDialog);
    } // setupUi

    void retranslateUi(QWidget *SvnProgressDialog)
    {
        SvnProgressDialog->setWindowTitle(QString());
        buttonCancel->setText(tr2i18n("Cancel", nullptr));
        buttonOk->setText(tr2i18n("OK", nullptr));
    } // retranslateUi

};

namespace Ui {
    class SvnProgressDialog: public Ui_SvnProgressDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // SVNPROGRESSDIALOG_H

