// This file was generated by ecm_qt_declare_logging_category(): DO NOT EDIT!

#include "mountedisooverlay_debug.h"


Q_LOGGING_CATEGORY(lcMountedIsoOverlay, "org.kde.dolphin.plugins.mountedisooverlay", QtInfoMsg)

