/********************************************************************************
** Form generated from reading UI file 'svncleanupdialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SVNCLEANUPDIALOG_H
#define UI_SVNCLEANUPDIALOG_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_SvnCleanupDialog
{
public:
    QGridLayout *gridLayout;
    QLabel *label;
    QLineEdit *lineEditDirectory;
    QCheckBox *checkBoxUnversioned;
    QCheckBox *checkBoxIgnored;
    QCheckBox *checkBoxExternals;
    QSpacerItem *horizontalSpacer;
    QPushButton *buttonOk;
    QPushButton *buttonCancel;

    void setupUi(QWidget *SvnCleanupDialog)
    {
        if (SvnCleanupDialog->objectName().isEmpty())
            SvnCleanupDialog->setObjectName("SvnCleanupDialog");
        SvnCleanupDialog->resize(370, 182);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Minimum);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(SvnCleanupDialog->sizePolicy().hasHeightForWidth());
        SvnCleanupDialog->setSizePolicy(sizePolicy);
        gridLayout = new QGridLayout(SvnCleanupDialog);
        gridLayout->setObjectName("gridLayout");
        label = new QLabel(SvnCleanupDialog);
        label->setObjectName("label");

        gridLayout->addWidget(label, 0, 0, 1, 1);

        lineEditDirectory = new QLineEdit(SvnCleanupDialog);
        lineEditDirectory->setObjectName("lineEditDirectory");
        lineEditDirectory->setEnabled(true);
        lineEditDirectory->setReadOnly(false);

        gridLayout->addWidget(lineEditDirectory, 1, 0, 1, 3);

        checkBoxUnversioned = new QCheckBox(SvnCleanupDialog);
        checkBoxUnversioned->setObjectName("checkBoxUnversioned");

        gridLayout->addWidget(checkBoxUnversioned, 2, 0, 1, 3);

        checkBoxIgnored = new QCheckBox(SvnCleanupDialog);
        checkBoxIgnored->setObjectName("checkBoxIgnored");

        gridLayout->addWidget(checkBoxIgnored, 3, 0, 1, 2);

        checkBoxExternals = new QCheckBox(SvnCleanupDialog);
        checkBoxExternals->setObjectName("checkBoxExternals");

        gridLayout->addWidget(checkBoxExternals, 4, 0, 1, 1);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(horizontalSpacer, 5, 0, 1, 1);

        buttonOk = new QPushButton(SvnCleanupDialog);
        buttonOk->setObjectName("buttonOk");
        buttonOk->setEnabled(false);
        QIcon icon(QIcon::fromTheme(QString::fromUtf8("dialog-ok")));
        buttonOk->setIcon(icon);
        buttonOk->setAutoDefault(true);

        gridLayout->addWidget(buttonOk, 5, 1, 1, 1);

        buttonCancel = new QPushButton(SvnCleanupDialog);
        buttonCancel->setObjectName("buttonCancel");
        QIcon icon1(QIcon::fromTheme(QString::fromUtf8("dialog-cancel")));
        buttonCancel->setIcon(icon1);

        gridLayout->addWidget(buttonCancel, 5, 2, 1, 1);


        retranslateUi(SvnCleanupDialog);

        buttonOk->setDefault(true);


        QMetaObject::connectSlotsByName(SvnCleanupDialog);
    } // setupUi

    void retranslateUi(QWidget *SvnCleanupDialog)
    {
        SvnCleanupDialog->setWindowTitle(tr2i18n("SVN Cleanup...", nullptr));
        label->setText(tr2i18n("Clean up directory:", nullptr));
        checkBoxUnversioned->setText(tr2i18n("Delete unversioned files and directories", nullptr));
        checkBoxIgnored->setText(tr2i18n("Delete ignored files and directories", nullptr));
        checkBoxExternals->setText(tr2i18n("Include externals", nullptr));
        buttonOk->setText(tr2i18n("OK", nullptr));
        buttonCancel->setText(tr2i18n("Cancel", nullptr));
    } // retranslateUi

};

namespace Ui {
    class SvnCleanupDialog: public Ui_SvnCleanupDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // SVNCLEANUPDIALOG_H

