/****************************************************************************
** Meta object code from reading C++ file 'fileviewsvnplugin.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../dolphin-plugins-26.08.0/svn/fileviewsvnplugin.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'fileviewsvnplugin.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN17FileViewSvnPluginE_t {};
} // unnamed namespace

template <> constexpr inline auto FileViewSvnPlugin::qt_create_metaobjectdata<qt_meta_tag_ZN17FileViewSvnPluginE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "FileViewSvnPlugin",
        "setShowUpdatesChecked",
        "",
        "checked",
        "versionInfoUpdated",
        "updateFiles",
        "showLocalChanges",
        "commitDialog",
        "addFiles",
        "removeFiles",
        "revertFiles",
        "logDialog",
        "checkoutDialog",
        "cleanupDialog",
        "slotOperationCompleted",
        "exitCode",
        "QProcess::ExitStatus",
        "exitStatus",
        "slotOperationError",
        "slotShowUpdatesToggled",
        "filesPath",
        "diffFile",
        "filePath",
        "diffAgainstWorkingCopy",
        "localFilePath",
        "rev",
        "diffBetweenRevs",
        "remoteFilePath",
        "rev1",
        "rev2",
        "commitFiles",
        "context",
        "msg"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'setShowUpdatesChecked'
        QtMocHelpers::SignalData<void(bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'versionInfoUpdated'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'updateFiles'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'showLocalChanges'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'commitDialog'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'addFiles'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'removeFiles'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'revertFiles'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'logDialog'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'checkoutDialog'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'cleanupDialog'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotOperationCompleted'
        QtMocHelpers::SlotData<void(int, QProcess::ExitStatus)>(14, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 15 }, { 0x80000000 | 16, 17 },
        }}),
        // Slot 'slotOperationError'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotShowUpdatesToggled'
        QtMocHelpers::SlotData<void(bool)>(19, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'revertFiles'
        QtMocHelpers::SlotData<void(const QStringList &)>(10, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QStringList, 20 },
        }}),
        // Slot 'diffFile'
        QtMocHelpers::SlotData<void(const QString &)>(21, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 22 },
        }}),
        // Slot 'diffAgainstWorkingCopy'
        QtMocHelpers::SlotData<void(const QString &, ulong)>(23, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 24 }, { QMetaType::ULong, 25 },
        }}),
        // Slot 'diffBetweenRevs'
        QtMocHelpers::SlotData<void(const QString &, ulong, ulong)>(26, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 27 }, { QMetaType::ULong, 28 }, { QMetaType::ULong, 29 },
        }}),
        // Slot 'addFiles'
        QtMocHelpers::SlotData<void(const QStringList &)>(8, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QStringList, 20 },
        }}),
        // Slot 'commitFiles'
        QtMocHelpers::SlotData<void(const QStringList &, const QString &)>(30, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QStringList, 31 }, { QMetaType::QString, 32 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<FileViewSvnPlugin, qt_meta_tag_ZN17FileViewSvnPluginE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject FileViewSvnPlugin::staticMetaObject = { {
    QMetaObject::SuperData::link<KVersionControlPlugin::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17FileViewSvnPluginE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17FileViewSvnPluginE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN17FileViewSvnPluginE_t>.metaTypes,
    nullptr
} };

void FileViewSvnPlugin::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<FileViewSvnPlugin *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->setShowUpdatesChecked((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 1: _t->versionInfoUpdated(); break;
        case 2: _t->updateFiles(); break;
        case 3: _t->showLocalChanges(); break;
        case 4: _t->commitDialog(); break;
        case 5: _t->addFiles(); break;
        case 6: _t->removeFiles(); break;
        case 7: _t->revertFiles(); break;
        case 8: _t->logDialog(); break;
        case 9: _t->checkoutDialog(); break;
        case 10: _t->cleanupDialog(); break;
        case 11: _t->slotOperationCompleted((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QProcess::ExitStatus>>(_a[2]))); break;
        case 12: _t->slotOperationError(); break;
        case 13: _t->slotShowUpdatesToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 14: _t->revertFiles((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 15: _t->diffFile((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 16: _t->diffAgainstWorkingCopy((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<ulong>>(_a[2]))); break;
        case 17: _t->diffBetweenRevs((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<ulong>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<ulong>>(_a[3]))); break;
        case 18: _t->addFiles((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 19: _t->commitFiles((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (FileViewSvnPlugin::*)(bool )>(_a, &FileViewSvnPlugin::setShowUpdatesChecked, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (FileViewSvnPlugin::*)()>(_a, &FileViewSvnPlugin::versionInfoUpdated, 1))
            return;
    }
}

const QMetaObject *FileViewSvnPlugin::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *FileViewSvnPlugin::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17FileViewSvnPluginE_t>.strings))
        return static_cast<void*>(this);
    return KVersionControlPlugin::qt_metacast(_clname);
}

int FileViewSvnPlugin::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KVersionControlPlugin::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 20)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 20;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 20)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 20;
    }
    return _id;
}

// SIGNAL 0
void FileViewSvnPlugin::setShowUpdatesChecked(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void FileViewSvnPlugin::versionInfoUpdated()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
QT_WARNING_POP
