// This file is generated by kconfig_compiler_kf6 from fileviewgitpluginsettings.kcfg.
// All changes you do to this file will be lost.

#include "fileviewgitpluginsettings.h"

#include <qglobal.h>
#include <QFile>

class FileViewGitPluginSettingsHelper
{
  public:
    FileViewGitPluginSettingsHelper() noexcept : q(nullptr) {}
    ~FileViewGitPluginSettingsHelper() noexcept { delete q; q = nullptr; }
    FileViewGitPluginSettingsHelper(const FileViewGitPluginSettingsHelper&) = delete;
    FileViewGitPluginSettingsHelper& operator=(const FileViewGitPluginSettingsHelper&) = delete;
    FileViewGitPluginSettings *q;
};
Q_GLOBAL_STATIC(FileViewGitPluginSettingsHelper, s_globalFileViewGitPluginSettings)
FileViewGitPluginSettings *FileViewGitPluginSettings::self()
{
  if (!s_globalFileViewGitPluginSettings()->q) {
    new FileViewGitPluginSettings;
    s_globalFileViewGitPluginSettings()->q->read();
  }

  return s_globalFileViewGitPluginSettings()->q;
}

FileViewGitPluginSettings::FileViewGitPluginSettings( )
  : KConfigSkeleton( QStringLiteral( "fileviewgitpluginrc" ) )
{
  Q_ASSERT(!s_globalFileViewGitPluginSettings()->q);
  s_globalFileViewGitPluginSettings()->q = this;
  setCurrentGroup( QStringLiteral( "CommitDialogSettings" ) );

  KConfigSkeleton::ItemInt  *itemCommitDialogHeight;
  itemCommitDialogHeight = new KConfigSkeleton::ItemInt( currentGroup(), QStringLiteral( "commitDialogHeight" ), mCommitDialogHeight, 300 );
  itemCommitDialogHeight->setMinValue(50);
  addItem( itemCommitDialogHeight, QStringLiteral( "commitDialogHeight" ) );
  KConfigSkeleton::ItemInt  *itemCommitDialogWidth;
  itemCommitDialogWidth = new KConfigSkeleton::ItemInt( currentGroup(), QStringLiteral( "commitDialogWidth" ), mCommitDialogWidth, 500 );
  itemCommitDialogWidth->setMinValue(50);
  addItem( itemCommitDialogWidth, QStringLiteral( "commitDialogWidth" ) );
}

FileViewGitPluginSettings::~FileViewGitPluginSettings()
{
  if (s_globalFileViewGitPluginSettings.exists() && !s_globalFileViewGitPluginSettings.isDestroyed()) {
    s_globalFileViewGitPluginSettings()->q = nullptr;
  }
}

