/****************************************************************************
** Meta object code from reading C++ file 'svnlogdialog.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../dolphin-plugins-26.08.0/svn/svnlogdialog.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'svnlogdialog.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN12SvnLogDialogE_t {};
} // unnamed namespace

template <> constexpr inline auto SvnLogDialog::qt_create_metaobjectdata<qt_meta_tag_ZN12SvnLogDialogE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "SvnLogDialog",
        "errorMessage",
        "",
        "msg",
        "operationCompletedMessage",
        "diffAgainstWorkingCopy",
        "localFilePath",
        "rev",
        "diffBetweenRevs",
        "remoteFilePath",
        "rev1",
        "rev2",
        "setCurrentRevision",
        "revision",
        "refreshLog",
        "on_tLog_currentCellChanged",
        "currentRow",
        "currentColumn",
        "previousRow",
        "previousColumn",
        "showContextMenuLog",
        "QPoint",
        "pos",
        "showContextMenuChangesList",
        "updateRepoToRevision",
        "revertRepoToRevision",
        "revertFileToRevision"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'errorMessage'
        QtMocHelpers::SignalData<void(const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'operationCompletedMessage'
        QtMocHelpers::SignalData<void(const QString &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'diffAgainstWorkingCopy'
        QtMocHelpers::SignalData<void(const QString &, ulong)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 }, { QMetaType::ULong, 7 },
        }}),
        // Signal 'diffBetweenRevs'
        QtMocHelpers::SignalData<void(const QString &, ulong, ulong)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 9 }, { QMetaType::ULong, 10 }, { QMetaType::ULong, 11 },
        }}),
        // Slot 'setCurrentRevision'
        QtMocHelpers::SlotData<void(ulong)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::ULong, 13 },
        }}),
        // Slot 'refreshLog'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_tLog_currentCellChanged'
        QtMocHelpers::SlotData<void(int, int, int, int)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 16 }, { QMetaType::Int, 17 }, { QMetaType::Int, 18 }, { QMetaType::Int, 19 },
        }}),
        // Slot 'showContextMenuLog'
        QtMocHelpers::SlotData<void(const QPoint &)>(20, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 21, 22 },
        }}),
        // Slot 'showContextMenuChangesList'
        QtMocHelpers::SlotData<void(const QPoint &)>(23, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 21, 22 },
        }}),
        // Slot 'updateRepoToRevision'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'revertRepoToRevision'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'revertFileToRevision'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<SvnLogDialog, qt_meta_tag_ZN12SvnLogDialogE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject SvnLogDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12SvnLogDialogE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12SvnLogDialogE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12SvnLogDialogE_t>.metaTypes,
    nullptr
} };

void SvnLogDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<SvnLogDialog *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->errorMessage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->operationCompletedMessage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->diffAgainstWorkingCopy((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<ulong>>(_a[2]))); break;
        case 3: _t->diffBetweenRevs((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<ulong>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<ulong>>(_a[3]))); break;
        case 4: _t->setCurrentRevision((*reinterpret_cast<std::add_pointer_t<ulong>>(_a[1]))); break;
        case 5: _t->refreshLog(); break;
        case 6: _t->on_tLog_currentCellChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[4]))); break;
        case 7: _t->showContextMenuLog((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 8: _t->showContextMenuChangesList((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 9: _t->updateRepoToRevision(); break;
        case 10: _t->revertRepoToRevision(); break;
        case 11: _t->revertFileToRevision(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (SvnLogDialog::*)(const QString & )>(_a, &SvnLogDialog::errorMessage, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (SvnLogDialog::*)(const QString & )>(_a, &SvnLogDialog::operationCompletedMessage, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (SvnLogDialog::*)(const QString & , ulong )>(_a, &SvnLogDialog::diffAgainstWorkingCopy, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (SvnLogDialog::*)(const QString & , ulong , ulong )>(_a, &SvnLogDialog::diffBetweenRevs, 3))
            return;
    }
}

const QMetaObject *SvnLogDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SvnLogDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12SvnLogDialogE_t>.strings))
        return static_cast<void*>(this);
    return QDialog::qt_metacast(_clname);
}

int SvnLogDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 12;
    }
    return _id;
}

// SIGNAL 0
void SvnLogDialog::errorMessage(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void SvnLogDialog::operationCompletedMessage(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void SvnLogDialog::diffAgainstWorkingCopy(const QString & _t1, ulong _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void SvnLogDialog::diffBetweenRevs(const QString & _t1, ulong _t2, ulong _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2, _t3);
}
QT_WARNING_POP
