// This file is generated by kconfig_compiler_kf6 from fileviewgitpluginsettings.kcfg.
// All changes you do to this file will be lost.
#ifndef FILEVIEWGITPLUGINSETTINGS_H
#define FILEVIEWGITPLUGINSETTINGS_H

#include <kconfigskeleton.h>
#include <QCoreApplication>
#include <QDebug>

class FileViewGitPluginSettings : public KConfigSkeleton
{
  public:

    static FileViewGitPluginSettings *self();
    ~FileViewGitPluginSettings() override;

    /**
      Set Dialog height
    */
    static
    void setCommitDialogHeight( int v )
    {
      if (v < 50)
      {
        qDebug() << "setCommitDialogHeight: value " << v << " is less than the minimum value of 50";
        v = 50;
      }
      if (!self()->isCommitDialogHeightImmutable())
        self()->mCommitDialogHeight = v;
    }

    /**
      Get Dialog height
    */
    static
    int commitDialogHeight()
    {
      return self()->mCommitDialogHeight;
    }

    /**
      Is Dialog height Immutable
    */
    static
    bool isCommitDialogHeightImmutable()
    {
      return self()->isImmutable( QStringLiteral( "commitDialogHeight" ) );
    }

    /**
      Set Dialog width
    */
    static
    void setCommitDialogWidth( int v )
    {
      if (v < 50)
      {
        qDebug() << "setCommitDialogWidth: value " << v << " is less than the minimum value of 50";
        v = 50;
      }
      if (!self()->isCommitDialogWidthImmutable())
        self()->mCommitDialogWidth = v;
    }

    /**
      Get Dialog width
    */
    static
    int commitDialogWidth()
    {
      return self()->mCommitDialogWidth;
    }

    /**
      Is Dialog width Immutable
    */
    static
    bool isCommitDialogWidthImmutable()
    {
      return self()->isImmutable( QStringLiteral( "commitDialogWidth" ) );
    }

  protected:
    FileViewGitPluginSettings();
    friend class FileViewGitPluginSettingsHelper;


    // CommitDialogSettings
    int mCommitDialogHeight;
    int mCommitDialogWidth;
};

#endif

