/********************************************************************************
** Form generated from reading UI file 'svncheckoutdialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SVNCHECKOUTDIALOG_H
#define UI_SVNCHECKOUTDIALOG_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_SvnCheckoutDialog
{
public:
    QVBoxLayout *verticalLayout;
    QLabel *label;
    QLineEdit *leRepository;
    QLabel *label_2;
    QLineEdit *leCheckoutDir;
    QCheckBox *cbOmitExternals;
    QHBoxLayout *horizontalLayout;
    QSpacerItem *horizontalSpacer;
    QPushButton *pbOk;
    QPushButton *pbCancel;

    void setupUi(QWidget *SvnCheckoutDialog)
    {
        if (SvnCheckoutDialog->objectName().isEmpty())
            SvnCheckoutDialog->setObjectName("SvnCheckoutDialog");
        SvnCheckoutDialog->resize(340, 180);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Minimum);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(SvnCheckoutDialog->sizePolicy().hasHeightForWidth());
        SvnCheckoutDialog->setSizePolicy(sizePolicy);
        verticalLayout = new QVBoxLayout(SvnCheckoutDialog);
        verticalLayout->setObjectName("verticalLayout");
        label = new QLabel(SvnCheckoutDialog);
        label->setObjectName("label");

        verticalLayout->addWidget(label);

        leRepository = new QLineEdit(SvnCheckoutDialog);
        leRepository->setObjectName("leRepository");

        verticalLayout->addWidget(leRepository);

        label_2 = new QLabel(SvnCheckoutDialog);
        label_2->setObjectName("label_2");

        verticalLayout->addWidget(label_2);

        leCheckoutDir = new QLineEdit(SvnCheckoutDialog);
        leCheckoutDir->setObjectName("leCheckoutDir");

        verticalLayout->addWidget(leCheckoutDir);

        cbOmitExternals = new QCheckBox(SvnCheckoutDialog);
        cbOmitExternals->setObjectName("cbOmitExternals");

        verticalLayout->addWidget(cbOmitExternals);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalSpacer = new QSpacerItem(148, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        pbOk = new QPushButton(SvnCheckoutDialog);
        pbOk->setObjectName("pbOk");
        pbOk->setEnabled(false);
        QIcon icon(QIcon::fromTheme(QString::fromUtf8("dialog-ok")));
        pbOk->setIcon(icon);

        horizontalLayout->addWidget(pbOk);

        pbCancel = new QPushButton(SvnCheckoutDialog);
        pbCancel->setObjectName("pbCancel");
        QIcon icon1(QIcon::fromTheme(QString::fromUtf8("dialog-cancel")));
        pbCancel->setIcon(icon1);

        horizontalLayout->addWidget(pbCancel);


        verticalLayout->addLayout(horizontalLayout);


        retranslateUi(SvnCheckoutDialog);

        QMetaObject::connectSlotsByName(SvnCheckoutDialog);
    } // setupUi

    void retranslateUi(QWidget *SvnCheckoutDialog)
    {
        SvnCheckoutDialog->setWindowTitle(tr2i18n("SVN Checkout", nullptr));
        label->setText(tr2i18n("URL of repository:", nullptr));
        label_2->setText(tr2i18n("Checkout directory:", nullptr));
        cbOmitExternals->setText(tr2i18n("Omit externals", nullptr));
        pbOk->setText(tr2i18n("OK", nullptr));
        pbCancel->setText(tr2i18n("Cancel", nullptr));
    } // retranslateUi

};

namespace Ui {
    class SvnCheckoutDialog: public Ui_SvnCheckoutDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // SVNCHECKOUTDIALOG_H

