// This file is generated by kconfig_compiler_kf6 from fileviewsvnpluginsettings.kcfg.
// All changes you do to this file will be lost.
#ifndef FILEVIEWSVNPLUGINSETTINGS_H
#define FILEVIEWSVNPLUGINSETTINGS_H

#include <kconfigskeleton.h>
#include <QCoreApplication>
#include <QDebug>

class FileViewSvnPluginSettings : public KConfigSkeleton
{
  public:

    static FileViewSvnPluginSettings *self();
    ~FileViewSvnPluginSettings() override;

    /**
      Set Show updates
    */
    static
    void setShowUpdates( bool v )
    {
      if (!self()->isShowUpdatesImmutable())
        self()->mShowUpdates = v;
    }

    /**
      Get Show updates
    */
    static
    bool showUpdates()
    {
      return self()->mShowUpdates;
    }

    /**
      Is Show updates Immutable
    */
    static
    bool isShowUpdatesImmutable()
    {
      return self()->isImmutable( QStringLiteral( "showUpdates" ) );
    }

  protected:
    FileViewSvnPluginSettings();
    friend class FileViewSvnPluginSettingsHelper;


    // General
    bool mShowUpdates;
};

#endif

