// This file is generated by kconfig_compiler_kf6 from fileviewhgpluginsettings.kcfg.
// All changes you do to this file will be lost.
#ifndef FILEVIEWHGPLUGINSETTINGS_H
#define FILEVIEWHGPLUGINSETTINGS_H

#include <kconfigskeleton.h>
#include <QCoreApplication>
#include <QDebug>

class FileViewHgPluginSettings : public KConfigSkeleton
{
  public:

    static FileViewHgPluginSettings *self();
    ~FileViewHgPluginSettings() override;

    /**
      Set Dialog height
    */
    static
    void setCommitDialogHeight( int v )
    {
      if (v < 400)
      {
        qDebug() << "setCommitDialogHeight: value " << v << " is less than the minimum value of 400";
        v = 400;
      }
      if (!self()->isCommitDialogHeightImmutable())
        self()->mCommitDialogHeight = v;
    }

    /**
      Get Dialog height
    */
    static
    int commitDialogHeight()
    {
      return self()->mCommitDialogHeight;
    }

    /**
      Is Dialog height Immutable
    */
    static
    bool isCommitDialogHeightImmutable()
    {
      return self()->isImmutable( QStringLiteral( "commitDialogHeight" ) );
    }

    /**
      Set Dialog width
    */
    static
    void setCommitDialogWidth( int v )
    {
      if (v < 500)
      {
        qDebug() << "setCommitDialogWidth: value " << v << " is less than the minimum value of 500";
        v = 500;
      }
      if (!self()->isCommitDialogWidthImmutable())
        self()->mCommitDialogWidth = v;
    }

    /**
      Get Dialog width
    */
    static
    int commitDialogWidth()
    {
      return self()->mCommitDialogWidth;
    }

    /**
      Is Dialog width Immutable
    */
    static
    bool isCommitDialogWidthImmutable()
    {
      return self()->isImmutable( QStringLiteral( "commitDialogWidth" ) );
    }

    /**
      Set Divides file list and editors with commit details
    */
    static
    void setVerticalSplitterSizes( const QList<int> & v )
    {
      if (!self()->isVerticalSplitterSizesImmutable())
        self()->mVerticalSplitterSizes = v;
    }

    /**
      Get Divides file list and editors with commit details
    */
    static
    QList<int> verticalSplitterSizes()
    {
      return self()->mVerticalSplitterSizes;
    }

    /**
      Is Divides file list and editors with commit details Immutable
    */
    static
    bool isVerticalSplitterSizesImmutable()
    {
      return self()->isImmutable( QStringLiteral( "verticalSplitterSizes" ) );
    }

    /**
      Set Divides commit editor and diff editor
    */
    static
    void setHorizontalSplitterSizes( const QList<int> & v )
    {
      if (!self()->isHorizontalSplitterSizesImmutable())
        self()->mHorizontalSplitterSizes = v;
    }

    /**
      Get Divides commit editor and diff editor
    */
    static
    QList<int> horizontalSplitterSizes()
    {
      return self()->mHorizontalSplitterSizes;
    }

    /**
      Is Divides commit editor and diff editor Immutable
    */
    static
    bool isHorizontalSplitterSizesImmutable()
    {
      return self()->isImmutable( QStringLiteral( "horizontalSplitterSizes" ) );
    }

    /**
      Set Dialog height
    */
    static
    void setConfigDialogHeight( int v )
    {
      if (v < 400)
      {
        qDebug() << "setConfigDialogHeight: value " << v << " is less than the minimum value of 400";
        v = 400;
      }
      if (!self()->isConfigDialogHeightImmutable())
        self()->mConfigDialogHeight = v;
    }

    /**
      Get Dialog height
    */
    static
    int configDialogHeight()
    {
      return self()->mConfigDialogHeight;
    }

    /**
      Is Dialog height Immutable
    */
    static
    bool isConfigDialogHeightImmutable()
    {
      return self()->isImmutable( QStringLiteral( "configDialogHeight" ) );
    }

    /**
      Set Dialog width
    */
    static
    void setConfigDialogWidth( int v )
    {
      if (v < 500)
      {
        qDebug() << "setConfigDialogWidth: value " << v << " is less than the minimum value of 500";
        v = 500;
      }
      if (!self()->isConfigDialogWidthImmutable())
        self()->mConfigDialogWidth = v;
    }

    /**
      Get Dialog width
    */
    static
    int configDialogWidth()
    {
      return self()->mConfigDialogWidth;
    }

    /**
      Is Dialog width Immutable
    */
    static
    bool isConfigDialogWidthImmutable()
    {
      return self()->isImmutable( QStringLiteral( "configDialogWidth" ) );
    }

    /**
      Set Dialog height
    */
    static
    void setCloneDialogHeight( int v )
    {
      if (!self()->isCloneDialogHeightImmutable())
        self()->mCloneDialogHeight = v;
    }

    /**
      Get Dialog height
    */
    static
    int cloneDialogHeight()
    {
      return self()->mCloneDialogHeight;
    }

    /**
      Is Dialog height Immutable
    */
    static
    bool isCloneDialogHeightImmutable()
    {
      return self()->isImmutable( QStringLiteral( "cloneDialogHeight" ) );
    }

    /**
      Set Dialog width
    */
    static
    void setCloneDialogWidth( int v )
    {
      if (!self()->isCloneDialogWidthImmutable())
        self()->mCloneDialogWidth = v;
    }

    /**
      Get Dialog width
    */
    static
    int cloneDialogWidth()
    {
      return self()->mCloneDialogWidth;
    }

    /**
      Is Dialog width Immutable
    */
    static
    bool isCloneDialogWidthImmutable()
    {
      return self()->isImmutable( QStringLiteral( "cloneDialogWidth" ) );
    }

    /**
      Set Dialog height
    */
    static
    void setPushDialogBigHeight( int v )
    {
      if (!self()->isPushDialogBigHeightImmutable())
        self()->mPushDialogBigHeight = v;
    }

    /**
      Get Dialog height
    */
    static
    int pushDialogBigHeight()
    {
      return self()->mPushDialogBigHeight;
    }

    /**
      Is Dialog height Immutable
    */
    static
    bool isPushDialogBigHeightImmutable()
    {
      return self()->isImmutable( QStringLiteral( "pushDialogBigHeight" ) );
    }

    /**
      Set Dialog width
    */
    static
    void setPushDialogBigWidth( int v )
    {
      if (!self()->isPushDialogBigWidthImmutable())
        self()->mPushDialogBigWidth = v;
    }

    /**
      Get Dialog width
    */
    static
    int pushDialogBigWidth()
    {
      return self()->mPushDialogBigWidth;
    }

    /**
      Is Dialog width Immutable
    */
    static
    bool isPushDialogBigWidthImmutable()
    {
      return self()->isImmutable( QStringLiteral( "pushDialogBigWidth" ) );
    }

    /**
      Set Dialog height
    */
    static
    void setPullDialogBigHeight( int v )
    {
      if (!self()->isPullDialogBigHeightImmutable())
        self()->mPullDialogBigHeight = v;
    }

    /**
      Get Dialog height
    */
    static
    int pullDialogBigHeight()
    {
      return self()->mPullDialogBigHeight;
    }

    /**
      Is Dialog height Immutable
    */
    static
    bool isPullDialogBigHeightImmutable()
    {
      return self()->isImmutable( QStringLiteral( "pullDialogBigHeight" ) );
    }

    /**
      Set Dialog width
    */
    static
    void setPullDialogBigWidth( int v )
    {
      if (!self()->isPullDialogBigWidthImmutable())
        self()->mPullDialogBigWidth = v;
    }

    /**
      Get Dialog width
    */
    static
    int pullDialogBigWidth()
    {
      return self()->mPullDialogBigWidth;
    }

    /**
      Is Dialog width Immutable
    */
    static
    bool isPullDialogBigWidthImmutable()
    {
      return self()->isImmutable( QStringLiteral( "pullDialogBigWidth" ) );
    }

    /**
      Set Dialog height
    */
    static
    void setMergeDialogHeight( int v )
    {
      if (!self()->isMergeDialogHeightImmutable())
        self()->mMergeDialogHeight = v;
    }

    /**
      Get Dialog height
    */
    static
    int mergeDialogHeight()
    {
      return self()->mMergeDialogHeight;
    }

    /**
      Is Dialog height Immutable
    */
    static
    bool isMergeDialogHeightImmutable()
    {
      return self()->isImmutable( QStringLiteral( "mergeDialogHeight" ) );
    }

    /**
      Set Dialog width
    */
    static
    void setMergeDialogWidth( int v )
    {
      if (!self()->isMergeDialogWidthImmutable())
        self()->mMergeDialogWidth = v;
    }

    /**
      Get Dialog width
    */
    static
    int mergeDialogWidth()
    {
      return self()->mMergeDialogWidth;
    }

    /**
      Is Dialog width Immutable
    */
    static
    bool isMergeDialogWidthImmutable()
    {
      return self()->isImmutable( QStringLiteral( "mergeDialogWidth" ) );
    }

    /**
      Set Dialog height
    */
    static
    void setBundleDialogHeight( int v )
    {
      if (!self()->isBundleDialogHeightImmutable())
        self()->mBundleDialogHeight = v;
    }

    /**
      Get Dialog height
    */
    static
    int bundleDialogHeight()
    {
      return self()->mBundleDialogHeight;
    }

    /**
      Is Dialog height Immutable
    */
    static
    bool isBundleDialogHeightImmutable()
    {
      return self()->isImmutable( QStringLiteral( "bundleDialogHeight" ) );
    }

    /**
      Set Dialog width
    */
    static
    void setBundleDialogWidth( int v )
    {
      if (!self()->isBundleDialogWidthImmutable())
        self()->mBundleDialogWidth = v;
    }

    /**
      Get Dialog width
    */
    static
    int bundleDialogWidth()
    {
      return self()->mBundleDialogWidth;
    }

    /**
      Is Dialog width Immutable
    */
    static
    bool isBundleDialogWidthImmutable()
    {
      return self()->isImmutable( QStringLiteral( "bundleDialogWidth" ) );
    }

    /**
      Set Dialog height
    */
    static
    void setExportDialogHeight( int v )
    {
      if (!self()->isExportDialogHeightImmutable())
        self()->mExportDialogHeight = v;
    }

    /**
      Get Dialog height
    */
    static
    int exportDialogHeight()
    {
      return self()->mExportDialogHeight;
    }

    /**
      Is Dialog height Immutable
    */
    static
    bool isExportDialogHeightImmutable()
    {
      return self()->isImmutable( QStringLiteral( "exportDialogHeight" ) );
    }

    /**
      Set Dialog width
    */
    static
    void setExportDialogWidth( int v )
    {
      if (!self()->isExportDialogWidthImmutable())
        self()->mExportDialogWidth = v;
    }

    /**
      Get Dialog width
    */
    static
    int exportDialogWidth()
    {
      return self()->mExportDialogWidth;
    }

    /**
      Is Dialog width Immutable
    */
    static
    bool isExportDialogWidthImmutable()
    {
      return self()->isImmutable( QStringLiteral( "exportDialogWidth" ) );
    }

    /**
      Set Dialog height
    */
    static
    void setImportDialogHeight( int v )
    {
      if (!self()->isImportDialogHeightImmutable())
        self()->mImportDialogHeight = v;
    }

    /**
      Get Dialog height
    */
    static
    int importDialogHeight()
    {
      return self()->mImportDialogHeight;
    }

    /**
      Is Dialog height Immutable
    */
    static
    bool isImportDialogHeightImmutable()
    {
      return self()->isImmutable( QStringLiteral( "importDialogHeight" ) );
    }

    /**
      Set Dialog width
    */
    static
    void setImportDialogWidth( int v )
    {
      if (!self()->isImportDialogWidthImmutable())
        self()->mImportDialogWidth = v;
    }

    /**
      Get Dialog width
    */
    static
    int importDialogWidth()
    {
      return self()->mImportDialogWidth;
    }

    /**
      Is Dialog width Immutable
    */
    static
    bool isImportDialogWidthImmutable()
    {
      return self()->isImmutable( QStringLiteral( "importDialogWidth" ) );
    }

    /**
      Set Dialog height
    */
    static
    void setServeDialogHeight( int v )
    {
      if (!self()->isServeDialogHeightImmutable())
        self()->mServeDialogHeight = v;
    }

    /**
      Get Dialog height
    */
    static
    int serveDialogHeight()
    {
      return self()->mServeDialogHeight;
    }

    /**
      Is Dialog height Immutable
    */
    static
    bool isServeDialogHeightImmutable()
    {
      return self()->isImmutable( QStringLiteral( "serveDialogHeight" ) );
    }

    /**
      Set Dialog width
    */
    static
    void setServeDialogWidth( int v )
    {
      if (!self()->isServeDialogWidthImmutable())
        self()->mServeDialogWidth = v;
    }

    /**
      Get Dialog width
    */
    static
    int serveDialogWidth()
    {
      return self()->mServeDialogWidth;
    }

    /**
      Is Dialog width Immutable
    */
    static
    bool isServeDialogWidthImmutable()
    {
      return self()->isImmutable( QStringLiteral( "serveDialogWidth" ) );
    }

    /**
      Set Dialog height
    */
    static
    void setBackoutDialogHeight( int v )
    {
      if (!self()->isBackoutDialogHeightImmutable())
        self()->mBackoutDialogHeight = v;
    }

    /**
      Get Dialog height
    */
    static
    int backoutDialogHeight()
    {
      return self()->mBackoutDialogHeight;
    }

    /**
      Is Dialog height Immutable
    */
    static
    bool isBackoutDialogHeightImmutable()
    {
      return self()->isImmutable( QStringLiteral( "backoutDialogHeight" ) );
    }

    /**
      Set Dialog width
    */
    static
    void setBackoutDialogWidth( int v )
    {
      if (!self()->isBackoutDialogWidthImmutable())
        self()->mBackoutDialogWidth = v;
    }

    /**
      Get Dialog width
    */
    static
    int backoutDialogWidth()
    {
      return self()->mBackoutDialogWidth;
    }

    /**
      Is Dialog width Immutable
    */
    static
    bool isBackoutDialogWidthImmutable()
    {
      return self()->isImmutable( QStringLiteral( "backoutDialogWidth" ) );
    }

  protected:
    FileViewHgPluginSettings();
    friend class FileViewHgPluginSettingsHelper;


    // CommitDialogSettings
    int mCommitDialogHeight;
    int mCommitDialogWidth;
    QList<int> mVerticalSplitterSizes;
    QList<int> mHorizontalSplitterSizes;

    // ConfigDialogSettings
    int mConfigDialogHeight;
    int mConfigDialogWidth;

    // CloneDialogSettings
    int mCloneDialogHeight;
    int mCloneDialogWidth;

    // PushDialogSettings
    int mPushDialogBigHeight;
    int mPushDialogBigWidth;

    // PullDialogSettings
    int mPullDialogBigHeight;
    int mPullDialogBigWidth;

    // MergeDialogSettings
    int mMergeDialogHeight;
    int mMergeDialogWidth;

    // BundleDialogSettings
    int mBundleDialogHeight;
    int mBundleDialogWidth;

    // ExportDialogSettings
    int mExportDialogHeight;
    int mExportDialogWidth;

    // ImportDialogSettings
    int mImportDialogHeight;
    int mImportDialogWidth;

    // ServeDialogSettings
    int mServeDialogHeight;
    int mServeDialogWidth;

    // BackoutDialogSettings
    int mBackoutDialogHeight;
    int mBackoutDialogWidth;
};

#endif

