// This file is generated by kconfig_compiler_kf6 from fileviewhgpluginsettings.kcfg.
// All changes you do to this file will be lost.

#include "fileviewhgpluginsettings.h"

#include <qglobal.h>
#include <QFile>

class FileViewHgPluginSettingsHelper
{
  public:
    FileViewHgPluginSettingsHelper() noexcept : q(nullptr) {}
    ~FileViewHgPluginSettingsHelper() noexcept { delete q; q = nullptr; }
    FileViewHgPluginSettingsHelper(const FileViewHgPluginSettingsHelper&) = delete;
    FileViewHgPluginSettingsHelper& operator=(const FileViewHgPluginSettingsHelper&) = delete;
    FileViewHgPluginSettings *q;
};
Q_GLOBAL_STATIC(FileViewHgPluginSettingsHelper, s_globalFileViewHgPluginSettings)
FileViewHgPluginSettings *FileViewHgPluginSettings::self()
{
  if (!s_globalFileViewHgPluginSettings()->q) {
    new FileViewHgPluginSettings;
    s_globalFileViewHgPluginSettings()->q->read();
  }

  return s_globalFileViewHgPluginSettings()->q;
}

FileViewHgPluginSettings::FileViewHgPluginSettings( )
  : KConfigSkeleton( QStringLiteral( "fileviewhgpluginrc" ) )
{
  Q_ASSERT(!s_globalFileViewHgPluginSettings()->q);
  s_globalFileViewHgPluginSettings()->q = this;
  setCurrentGroup( QStringLiteral( "CommitDialogSettings" ) );

  KConfigSkeleton::ItemInt  *itemCommitDialogHeight;
  itemCommitDialogHeight = new KConfigSkeleton::ItemInt( currentGroup(), QStringLiteral( "commitDialogHeight" ), mCommitDialogHeight, 550 );
  itemCommitDialogHeight->setMinValue(400);
  addItem( itemCommitDialogHeight, QStringLiteral( "commitDialogHeight" ) );
  KConfigSkeleton::ItemInt  *itemCommitDialogWidth;
  itemCommitDialogWidth = new KConfigSkeleton::ItemInt( currentGroup(), QStringLiteral( "commitDialogWidth" ), mCommitDialogWidth, 900 );
  itemCommitDialogWidth->setMinValue(500);
  addItem( itemCommitDialogWidth, QStringLiteral( "commitDialogWidth" ) );
  QList<int> defaultverticalSplitterSizes;
  defaultverticalSplitterSizes.append( 200 );
  defaultverticalSplitterSizes.append( 700 );

  KConfigSkeleton::ItemIntList  *itemVerticalSplitterSizes;
  itemVerticalSplitterSizes = new KConfigSkeleton::ItemIntList( currentGroup(), QStringLiteral( "verticalSplitterSizes" ), mVerticalSplitterSizes, defaultverticalSplitterSizes );
  addItem( itemVerticalSplitterSizes, QStringLiteral( "verticalSplitterSizes" ) );
  QList<int> defaulthorizontalSplitterSizes;
  defaulthorizontalSplitterSizes.append( 100 );
  defaulthorizontalSplitterSizes.append( 350 );

  KConfigSkeleton::ItemIntList  *itemHorizontalSplitterSizes;
  itemHorizontalSplitterSizes = new KConfigSkeleton::ItemIntList( currentGroup(), QStringLiteral( "horizontalSplitterSizes" ), mHorizontalSplitterSizes, defaulthorizontalSplitterSizes );
  addItem( itemHorizontalSplitterSizes, QStringLiteral( "horizontalSplitterSizes" ) );

  setCurrentGroup( QStringLiteral( "ConfigDialogSettings" ) );

  KConfigSkeleton::ItemInt  *itemConfigDialogHeight;
  itemConfigDialogHeight = new KConfigSkeleton::ItemInt( currentGroup(), QStringLiteral( "configDialogHeight" ), mConfigDialogHeight, 550 );
  itemConfigDialogHeight->setMinValue(400);
  addItem( itemConfigDialogHeight, QStringLiteral( "configDialogHeight" ) );
  KConfigSkeleton::ItemInt  *itemConfigDialogWidth;
  itemConfigDialogWidth = new KConfigSkeleton::ItemInt( currentGroup(), QStringLiteral( "configDialogWidth" ), mConfigDialogWidth, 900 );
  itemConfigDialogWidth->setMinValue(500);
  addItem( itemConfigDialogWidth, QStringLiteral( "configDialogWidth" ) );

  setCurrentGroup( QStringLiteral( "CloneDialogSettings" ) );

  KConfigSkeleton::ItemInt  *itemCloneDialogHeight;
  itemCloneDialogHeight = new KConfigSkeleton::ItemInt( currentGroup(), QStringLiteral( "cloneDialogHeight" ), mCloneDialogHeight, 550 );
  addItem( itemCloneDialogHeight, QStringLiteral( "cloneDialogHeight" ) );
  KConfigSkeleton::ItemInt  *itemCloneDialogWidth;
  itemCloneDialogWidth = new KConfigSkeleton::ItemInt( currentGroup(), QStringLiteral( "cloneDialogWidth" ), mCloneDialogWidth, 900 );
  addItem( itemCloneDialogWidth, QStringLiteral( "cloneDialogWidth" ) );

  setCurrentGroup( QStringLiteral( "PushDialogSettings" ) );

  KConfigSkeleton::ItemInt  *itemPushDialogBigHeight;
  itemPushDialogBigHeight = new KConfigSkeleton::ItemInt( currentGroup(), QStringLiteral( "pushDialogBigHeight" ), mPushDialogBigHeight, 550 );
  addItem( itemPushDialogBigHeight, QStringLiteral( "pushDialogBigHeight" ) );
  KConfigSkeleton::ItemInt  *itemPushDialogBigWidth;
  itemPushDialogBigWidth = new KConfigSkeleton::ItemInt( currentGroup(), QStringLiteral( "pushDialogBigWidth" ), mPushDialogBigWidth, 900 );
  addItem( itemPushDialogBigWidth, QStringLiteral( "pushDialogBigWidth" ) );

  setCurrentGroup( QStringLiteral( "PullDialogSettings" ) );

  KConfigSkeleton::ItemInt  *itemPullDialogBigHeight;
  itemPullDialogBigHeight = new KConfigSkeleton::ItemInt( currentGroup(), QStringLiteral( "pullDialogBigHeight" ), mPullDialogBigHeight, 550 );
  addItem( itemPullDialogBigHeight, QStringLiteral( "pullDialogBigHeight" ) );
  KConfigSkeleton::ItemInt  *itemPullDialogBigWidth;
  itemPullDialogBigWidth = new KConfigSkeleton::ItemInt( currentGroup(), QStringLiteral( "pullDialogBigWidth" ), mPullDialogBigWidth, 900 );
  addItem( itemPullDialogBigWidth, QStringLiteral( "pullDialogBigWidth" ) );

  setCurrentGroup( QStringLiteral( "MergeDialogSettings" ) );

  KConfigSkeleton::ItemInt  *itemMergeDialogHeight;
  itemMergeDialogHeight = new KConfigSkeleton::ItemInt( currentGroup(), QStringLiteral( "mergeDialogHeight" ), mMergeDialogHeight, 550 );
  addItem( itemMergeDialogHeight, QStringLiteral( "mergeDialogHeight" ) );
  KConfigSkeleton::ItemInt  *itemMergeDialogWidth;
  itemMergeDialogWidth = new KConfigSkeleton::ItemInt( currentGroup(), QStringLiteral( "mergeDialogWidth" ), mMergeDialogWidth, 900 );
  addItem( itemMergeDialogWidth, QStringLiteral( "mergeDialogWidth" ) );

  setCurrentGroup( QStringLiteral( "BundleDialogSettings" ) );

  KConfigSkeleton::ItemInt  *itemBundleDialogHeight;
  itemBundleDialogHeight = new KConfigSkeleton::ItemInt( currentGroup(), QStringLiteral( "bundleDialogHeight" ), mBundleDialogHeight, 550 );
  addItem( itemBundleDialogHeight, QStringLiteral( "bundleDialogHeight" ) );
  KConfigSkeleton::ItemInt  *itemBundleDialogWidth;
  itemBundleDialogWidth = new KConfigSkeleton::ItemInt( currentGroup(), QStringLiteral( "bundleDialogWidth" ), mBundleDialogWidth, 900 );
  addItem( itemBundleDialogWidth, QStringLiteral( "bundleDialogWidth" ) );

  setCurrentGroup( QStringLiteral( "ExportDialogSettings" ) );

  KConfigSkeleton::ItemInt  *itemExportDialogHeight;
  itemExportDialogHeight = new KConfigSkeleton::ItemInt( currentGroup(), QStringLiteral( "exportDialogHeight" ), mExportDialogHeight, 550 );
  addItem( itemExportDialogHeight, QStringLiteral( "exportDialogHeight" ) );
  KConfigSkeleton::ItemInt  *itemExportDialogWidth;
  itemExportDialogWidth = new KConfigSkeleton::ItemInt( currentGroup(), QStringLiteral( "exportDialogWidth" ), mExportDialogWidth, 900 );
  addItem( itemExportDialogWidth, QStringLiteral( "exportDialogWidth" ) );

  setCurrentGroup( QStringLiteral( "ImportDialogSettings" ) );

  KConfigSkeleton::ItemInt  *itemImportDialogHeight;
  itemImportDialogHeight = new KConfigSkeleton::ItemInt( currentGroup(), QStringLiteral( "importDialogHeight" ), mImportDialogHeight, 540 );
  addItem( itemImportDialogHeight, QStringLiteral( "importDialogHeight" ) );
  KConfigSkeleton::ItemInt  *itemImportDialogWidth;
  itemImportDialogWidth = new KConfigSkeleton::ItemInt( currentGroup(), QStringLiteral( "importDialogWidth" ), mImportDialogWidth, 500 );
  addItem( itemImportDialogWidth, QStringLiteral( "importDialogWidth" ) );

  setCurrentGroup( QStringLiteral( "ServeDialogSettings" ) );

  KConfigSkeleton::ItemInt  *itemServeDialogHeight;
  itemServeDialogHeight = new KConfigSkeleton::ItemInt( currentGroup(), QStringLiteral( "serveDialogHeight" ), mServeDialogHeight, 600 );
  addItem( itemServeDialogHeight, QStringLiteral( "serveDialogHeight" ) );
  KConfigSkeleton::ItemInt  *itemServeDialogWidth;
  itemServeDialogWidth = new KConfigSkeleton::ItemInt( currentGroup(), QStringLiteral( "serveDialogWidth" ), mServeDialogWidth, 500 );
  addItem( itemServeDialogWidth, QStringLiteral( "serveDialogWidth" ) );

  setCurrentGroup( QStringLiteral( "BackoutDialogSettings" ) );

  KConfigSkeleton::ItemInt  *itemBackoutDialogHeight;
  itemBackoutDialogHeight = new KConfigSkeleton::ItemInt( currentGroup(), QStringLiteral( "backoutDialogHeight" ), mBackoutDialogHeight, 346 );
  addItem( itemBackoutDialogHeight, QStringLiteral( "backoutDialogHeight" ) );
  KConfigSkeleton::ItemInt  *itemBackoutDialogWidth;
  itemBackoutDialogWidth = new KConfigSkeleton::ItemInt( currentGroup(), QStringLiteral( "backoutDialogWidth" ), mBackoutDialogWidth, 537 );
  addItem( itemBackoutDialogWidth, QStringLiteral( "backoutDialogWidth" ) );
}

FileViewHgPluginSettings::~FileViewHgPluginSettings()
{
  if (s_globalFileViewHgPluginSettings.exists() && !s_globalFileViewHgPluginSettings.isDestroyed()) {
    s_globalFileViewHgPluginSettings()->q = nullptr;
  }
}

