/****************************************************************************
** Meta object code from reading C++ file 'syncdialogbase.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../dolphin-plugins-26.08.0/hg/syncdialogbase.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'syncdialogbase.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN16HgSyncBaseDialogE_t {};
} // unnamed namespace

template <> constexpr inline auto HgSyncBaseDialog::qt_create_metaobjectdata<qt_meta_tag_ZN16HgSyncBaseDialogE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "HgSyncBaseDialog",
        "changeListAvailable",
        "",
        "slotGetChanges",
        "slotChangesProcessComplete",
        "exitCode",
        "QProcess::ExitStatus",
        "status",
        "slotChangesProcessError",
        "slotOperationComplete",
        "slotOperationError",
        "slotUpdateBusy",
        "QProcess::ProcessState",
        "state",
        "slotWriteBigSize",
        "slotOptionsButtonClick",
        "writeBigSize",
        "readBigSize"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'changeListAvailable'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotGetChanges'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotChangesProcessComplete'
        QtMocHelpers::SlotData<void(int, QProcess::ExitStatus)>(4, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 5 }, { 0x80000000 | 6, 7 },
        }}),
        // Slot 'slotChangesProcessError'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotOperationComplete'
        QtMocHelpers::SlotData<void(int, QProcess::ExitStatus)>(9, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 5 }, { 0x80000000 | 6, 7 },
        }}),
        // Slot 'slotOperationError'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotUpdateBusy'
        QtMocHelpers::SlotData<void(QProcess::ProcessState)>(11, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Slot 'slotWriteBigSize'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotOptionsButtonClick'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'writeBigSize'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'readBigSize'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessProtected, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<HgSyncBaseDialog, qt_meta_tag_ZN16HgSyncBaseDialogE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject HgSyncBaseDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<DialogBase::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16HgSyncBaseDialogE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16HgSyncBaseDialogE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16HgSyncBaseDialogE_t>.metaTypes,
    nullptr
} };

void HgSyncBaseDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<HgSyncBaseDialog *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->changeListAvailable(); break;
        case 1: _t->slotGetChanges(); break;
        case 2: _t->slotChangesProcessComplete((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QProcess::ExitStatus>>(_a[2]))); break;
        case 3: _t->slotChangesProcessError(); break;
        case 4: _t->slotOperationComplete((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QProcess::ExitStatus>>(_a[2]))); break;
        case 5: _t->slotOperationError(); break;
        case 6: _t->slotUpdateBusy((*reinterpret_cast<std::add_pointer_t<QProcess::ProcessState>>(_a[1]))); break;
        case 7: _t->slotWriteBigSize(); break;
        case 8: _t->slotOptionsButtonClick(); break;
        case 9: _t->writeBigSize(); break;
        case 10: _t->readBigSize(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (HgSyncBaseDialog::*)()>(_a, &HgSyncBaseDialog::changeListAvailable, 0))
            return;
    }
}

const QMetaObject *HgSyncBaseDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *HgSyncBaseDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16HgSyncBaseDialogE_t>.strings))
        return static_cast<void*>(this);
    return DialogBase::qt_metacast(_clname);
}

int HgSyncBaseDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DialogBase::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void HgSyncBaseDialog::changeListAvailable()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
QT_WARNING_POP
