/********************************************************************************
** Form generated from reading UI file 'svnlogdialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SVNLOGDIALOG_H
#define UI_SVNLOGDIALOG_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QFrame>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QPlainTextEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTableWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_SvnLogDialog
{
public:
    QVBoxLayout *verticalLayout;
    QTableWidget *tLog;
    QFrame *line;
    QPlainTextEdit *teMessage;
    QFrame *line_2;
    QListWidget *lPaths;
    QHBoxLayout *horizontalLayout;
    QPushButton *pbNext100;
    QPushButton *pbRefresh;
    QSpacerItem *horizontalSpacer;
    QPushButton *pbOk;

    void setupUi(QWidget *SvnLogDialog)
    {
        if (SvnLogDialog->objectName().isEmpty())
            SvnLogDialog->setObjectName("SvnLogDialog");
        SvnLogDialog->resize(726, 654);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Minimum);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(SvnLogDialog->sizePolicy().hasHeightForWidth());
        SvnLogDialog->setSizePolicy(sizePolicy);
        verticalLayout = new QVBoxLayout(SvnLogDialog);
        verticalLayout->setObjectName("verticalLayout");
        tLog = new QTableWidget(SvnLogDialog);
        if (tLog->columnCount() < 4)
            tLog->setColumnCount(4);
        QTableWidgetItem *__qtablewidgetitem = new QTableWidgetItem();
        tLog->setHorizontalHeaderItem(0, __qtablewidgetitem);
        QTableWidgetItem *__qtablewidgetitem1 = new QTableWidgetItem();
        tLog->setHorizontalHeaderItem(1, __qtablewidgetitem1);
        QTableWidgetItem *__qtablewidgetitem2 = new QTableWidgetItem();
        tLog->setHorizontalHeaderItem(2, __qtablewidgetitem2);
        QTableWidgetItem *__qtablewidgetitem3 = new QTableWidgetItem();
        tLog->setHorizontalHeaderItem(3, __qtablewidgetitem3);
        if (tLog->rowCount() < 1)
            tLog->setRowCount(1);
        tLog->setObjectName("tLog");
        tLog->setContextMenuPolicy(Qt::CustomContextMenu);
        tLog->setEditTriggers(QAbstractItemView::NoEditTriggers);
        tLog->setSelectionMode(QAbstractItemView::SingleSelection);
        tLog->setSelectionBehavior(QAbstractItemView::SelectRows);
        tLog->setCornerButtonEnabled(false);
        tLog->setRowCount(1);
        tLog->horizontalHeader()->setStretchLastSection(true);
        tLog->verticalHeader()->setVisible(false);
        tLog->verticalHeader()->setStretchLastSection(false);

        verticalLayout->addWidget(tLog);

        line = new QFrame(SvnLogDialog);
        line->setObjectName("line");
        line->setFrameShape(QFrame::Shape::HLine);
        line->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout->addWidget(line);

        teMessage = new QPlainTextEdit(SvnLogDialog);
        teMessage->setObjectName("teMessage");

        verticalLayout->addWidget(teMessage);

        line_2 = new QFrame(SvnLogDialog);
        line_2->setObjectName("line_2");
        line_2->setFrameShape(QFrame::Shape::HLine);
        line_2->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout->addWidget(line_2);

        lPaths = new QListWidget(SvnLogDialog);
        lPaths->setObjectName("lPaths");
        lPaths->setContextMenuPolicy(Qt::CustomContextMenu);

        verticalLayout->addWidget(lPaths);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        pbNext100 = new QPushButton(SvnLogDialog);
        pbNext100->setObjectName("pbNext100");
        QIcon icon(QIcon::fromTheme(QString::fromUtf8("go-down")));
        pbNext100->setIcon(icon);

        horizontalLayout->addWidget(pbNext100);

        pbRefresh = new QPushButton(SvnLogDialog);
        pbRefresh->setObjectName("pbRefresh");
        QIcon icon1(QIcon::fromTheme(QString::fromUtf8("view-refresh")));
        pbRefresh->setIcon(icon1);

        horizontalLayout->addWidget(pbRefresh);

        horizontalSpacer = new QSpacerItem(448, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        pbOk = new QPushButton(SvnLogDialog);
        pbOk->setObjectName("pbOk");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(pbOk->sizePolicy().hasHeightForWidth());
        pbOk->setSizePolicy(sizePolicy1);
        QIcon icon2(QIcon::fromTheme(QString::fromUtf8("dialog-ok")));
        pbOk->setIcon(icon2);

        horizontalLayout->addWidget(pbOk);


        verticalLayout->addLayout(horizontalLayout);

        verticalLayout->setStretch(0, 5);
        verticalLayout->setStretch(1, 1);
        verticalLayout->setStretch(2, 1);
        verticalLayout->setStretch(3, 1);
        verticalLayout->setStretch(4, 1);

        retranslateUi(SvnLogDialog);

        QMetaObject::connectSlotsByName(SvnLogDialog);
    } // setupUi

    void retranslateUi(QWidget *SvnLogDialog)
    {
        SvnLogDialog->setWindowTitle(tr2i18n("SVN Log", nullptr));
        QTableWidgetItem *___qtablewidgetitem = tLog->horizontalHeaderItem(0);
        ___qtablewidgetitem->setText(tr2i18n("Revision", nullptr));
        QTableWidgetItem *___qtablewidgetitem1 = tLog->horizontalHeaderItem(1);
        ___qtablewidgetitem1->setText(tr2i18n("Author", nullptr));
        QTableWidgetItem *___qtablewidgetitem2 = tLog->horizontalHeaderItem(2);
        ___qtablewidgetitem2->setText(tr2i18n("Date", nullptr));
        QTableWidgetItem *___qtablewidgetitem3 = tLog->horizontalHeaderItem(3);
        ___qtablewidgetitem3->setText(tr2i18n("Message", nullptr));
        pbNext100->setText(tr2i18n("Next 100", nullptr));
        pbRefresh->setText(tr2i18n("Refresh", nullptr));
        pbOk->setText(tr2i18n("OK", nullptr));
    } // retranslateUi

};

namespace Ui {
    class SvnLogDialog: public Ui_SvnLogDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // SVNLOGDIALOG_H

