/****************************************************************************
** Meta object code from reading C++ file 'fileviewhgplugin.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../dolphin-plugins-26.08.0/hg/fileviewhgplugin.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'fileviewhgplugin.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN16FileViewHgPluginE_t {};
} // unnamed namespace

template <> constexpr inline auto FileViewHgPlugin::qt_create_metaobjectdata<qt_meta_tag_ZN16FileViewHgPluginE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "FileViewHgPlugin",
        "addFiles",
        "",
        "removeFiles",
        "renameFile",
        "commit",
        "branch",
        "tag",
        "update",
        "clone",
        "create",
        "global_config",
        "repo_config",
        "push",
        "pull",
        "revert",
        "revertAll",
        "rollback",
        "backout",
        "diff",
        "serve",
        "merge",
        "bundle",
        "unbundle",
        "exportChangesets",
        "importChangesets",
        "slotOperationCompleted",
        "exitCode",
        "QProcess::ExitStatus",
        "exitStatus",
        "slotOperationError"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'addFiles'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'removeFiles'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'renameFile'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'commit'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'branch'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'tag'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'update'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'clone'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'create'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'global_config'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'repo_config'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'push'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'pull'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'revert'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'revertAll'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'rollback'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'backout'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'diff'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'serve'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'merge'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'bundle'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'unbundle'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'exportChangesets'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'importChangesets'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotOperationCompleted'
        QtMocHelpers::SlotData<void(int, QProcess::ExitStatus)>(26, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 27 }, { 0x80000000 | 28, 29 },
        }}),
        // Slot 'slotOperationError'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<FileViewHgPlugin, qt_meta_tag_ZN16FileViewHgPluginE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject FileViewHgPlugin::staticMetaObject = { {
    QMetaObject::SuperData::link<KVersionControlPlugin::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16FileViewHgPluginE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16FileViewHgPluginE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16FileViewHgPluginE_t>.metaTypes,
    nullptr
} };

void FileViewHgPlugin::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<FileViewHgPlugin *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->addFiles(); break;
        case 1: _t->removeFiles(); break;
        case 2: _t->renameFile(); break;
        case 3: _t->commit(); break;
        case 4: _t->branch(); break;
        case 5: _t->tag(); break;
        case 6: _t->update(); break;
        case 7: _t->clone(); break;
        case 8: _t->create(); break;
        case 9: _t->global_config(); break;
        case 10: _t->repo_config(); break;
        case 11: _t->push(); break;
        case 12: _t->pull(); break;
        case 13: _t->revert(); break;
        case 14: _t->revertAll(); break;
        case 15: _t->rollback(); break;
        case 16: _t->backout(); break;
        case 17: _t->diff(); break;
        case 18: _t->serve(); break;
        case 19: _t->merge(); break;
        case 20: _t->bundle(); break;
        case 21: _t->unbundle(); break;
        case 22: _t->exportChangesets(); break;
        case 23: _t->importChangesets(); break;
        case 24: _t->slotOperationCompleted((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QProcess::ExitStatus>>(_a[2]))); break;
        case 25: _t->slotOperationError(); break;
        default: ;
        }
    }
}

const QMetaObject *FileViewHgPlugin::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *FileViewHgPlugin::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16FileViewHgPluginE_t>.strings))
        return static_cast<void*>(this);
    return KVersionControlPlugin::qt_metacast(_clname);
}

int FileViewHgPlugin::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KVersionControlPlugin::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 26)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 26;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 26)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 26;
    }
    return _id;
}
QT_WARNING_POP
